#
# Makefile Template for Tessy
# Compiler: Microchip XC32
# Debugger: Microchip MPLAB X
# Template Version: 2
#

# Defines to be generated by Tessy:
#
# BUFFER_SIZE            :=
# CHIP                   :=
# COMPILER_DEFINES       :=
# COMPILER_INCLUDES      :=
# COMPILER_INSTALL_PATH  :=
# ENDIANESS              :=
# EXEFILE_EXTENSION      :=
# LINKER_FILE            :=
# MODULE_PATH            :=
# MODULE_PATH_DOS        :=
# OBJFILE_EXTENSION      :=
# PROJECTROOT            :=
# RESERVED_MEMORY        :=
# TARGET_INSTALL_PATH    :=
# TESSY_BIN              :=
# TESSY_SYS              :=
# TESTOBJECT             :=
# TIMER_FILE             :=
# USER_LINK_OPTIONS      :=

#####################################################################
#BEGIN_DEFINES
#####################################################################

#
# Special macros for includes and defines
#
TS_DEF_SEPARATOR :=-D
define TS_DEFINE
$(subst ;, $(TS_DEF_SEPARATOR),$(1))
endef

TS_INC_SEPARATOR :=-I 
define TS_INCLUDE
$(subst ;, $(TS_INC_SEPARATOR),$(1))
endef

# Chip          := 32MZ1024ECG064
TESSY_SYS_DOS   := $(shell ts_pathname -s '$(TESSY_SYS)')
MPLAB_HOME      := $(shell ts_pathname -s '$(COMPILER_INSTALL_PATH)')
LINK_FILE_SHORT := $(shell ts_pathname -s '$(LINKER_FILE)')

TS_NAME         := TS_MPLAB
IDE_NAME        := mplabx
COMPANY_NAME    := microchip
PROCESSOR_NAME  := xc32

ifeq ("little","$(ENDIANESS)")
  TS_BYTEORDER := TS_HOST_BYTEORDER
  TS_BYTEORDER_OPTION :=
endif
ifeq ("big","$(ENDIANESS)")
  TS_BYTEORDER := TS_NET_BYTEORDER
  TS_BYTEORDER_OPTION :=
endif
ifndef TS_BYTEORDER
  $(error Attribute 'Endianess' has invalid value '$(ENDIANESS)', only 'little' and 'big' are allowed.)
endif

all	 : check_success master slave

# Cygwin work around.
S_BUILDIN_TESSY_FLAG := $(wildcard $(MODULE_PATH)/tslows_builtin.tessy)
check_success :
ifneq ($(S_BUILDIN_TESSY_FLAG),)
	$(info *)
	$(info * Obviously the last build failed. Cleaning up built-in files...)
	@-ts_rm -f "$(MODULE_PATH)"/tslows_builtin.*
	$(info * Please, try again to generate a built-in target binary.)
	$(info *)
	@exit 1
endif

ifeq ($(ALIGN_COMM_BUFFER),)
ALIGN_COMM_BUFFER := 0
endif

LINKER_SCRIPT := -Wl,--script=
LINKER_HEAP   := -Wl,--defsym=_min_heap_size=
LINKER_STACK  := -Wl,--defsym=_min_stack_size=

#
# MASTER & COMM Makefiles
# (you should not need to modify these settings)
#
M_COMM_MAKE     := $(TESSY_SYS)/make/master_$(IDE_NAME)_gnu_i386.mk
S_COMM_MAKE     := $(TESSY_SYS)/src/comm/comm_$(IDE_NAME)_$(COMPANY_NAME)_$(PROCESSOR_NAME).mk

#
#
# SLAVE
#
S_CC            := $(MPLAB_HOME)\bin\xc32-g++.exe
S_LINK          := $(MPLAB_HOME)\bin\xc32-g++.exe
S_BIN2HEX       := $(MPLAB_HOME)\bin\xc32-bin2hex
S_INCLUDES      := -I $(TESSY_SYS_DOS)\include\tessy\comm -I $(MODULE_PATH_DOS)
S_STUB_OBJECT   := $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_stubs$(OBJFILE_EXTENSION)
S_UC_OBJECT     := $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_usr$(OBJFILE_EXTENSION)

# CPU = SAMA5D27
S_COMP_OPTIONS  := -c $(COMPILER_DIALECT) -mprocessor=$(CHIP) -MP -MMD -g -D__DEBUG -D__FPU_PRESENT=1 \
                   -fno-common -DXPRJ_default=default -mdfp="E:/Microchip/MPLABX/v5.50/packs/Microchip/SAMA5D2_DFP/1.5.53" \
                   -x c++ -O0 -Wall -DTESSY -DTS_SLAVE -D$(TS_NAME) -DTS_HAVE_INT64 -DTS_HAVE_FLOAT64 \
                   -DTS_MICROCHIP_XC32 -DTS_ALIGN_BUFFER=$(ALIGN_COMM_BUFFER) \
						 $(if $(findstr true,$(ENABLE_EXCEPTIONS)),-fexceptions)
S_LINK_OPTIONS  := -mdebugger -D__MPLAB_DEBUGGER_ICD3=1 -mprocessor=$(CHIP) -DXPRJ_default=default \
                   $(TS_BYTEORDER_OPTION) $(addprefix -mreserve=,$(RESERVED_MEMORY)) -Wl,--gc-sections\
                   -Wl,--defsym=__MPLAB_BUILD=1 $(addprefix $(LINKER_SCRIPT),$(LINK_FILE_SHORT)) \
                   $(addprefix $(LINKER_HEAP),$(HEAP_SIZE)) $(addprefix $(LINKER_STACK),$(STACK_SIZE)) \
						 $(if $(findstr true,$(ENABLE_EXCEPTIONS)),-fexceptions)

#####################################################################
#END_DEFINES
#####################################################################

#####################################################################
#BEGIN_SOURCE
#####################################################################

#
# Template rules
#
$(MODULE_PATH_DOS)\ts_src$$(SOURCENUMBER)$(OBJFILE_EXTENSION): $(MODULE_PATH_DOS)\ts_src$$(SOURCENUMBER).cpp
	$(S_CC) $(S_INCLUDES) $(call TS_INCLUDE,$(USER_INCLUDES$$(SOURCENUMBER))) $(S_COMP_OPTIONS) \
		$(call TS_DEFINE,$(USER_DEFINES$$(SOURCENUMBER))$(S_COM_OPTIONS)) \
		$(USER_COMP_OPTIONS$$(SOURCENUMBER)) -o "$@" "$<"


#####################################################################
#END_SOURCE
#####################################################################

#####################################################################
#BEGIN_TARGETS
#####################################################################

ifneq ($(S_GenerateBuiltinData),)
TARGET_FLAG := d
BUILTIN_OBJ := $(MODULE_PATH_DOS)\tslows_builtin$(OBJFILE_EXTENSION)
master :
	@-ts_rm -f "$(MODULE_PATH)"/tslows_builtin$(OBJFILE_EXTENSION)
else
BUILTIN_OBJ :=
TARGET_FLAG := s
master : $(MODULE_PATH)/ts_$(TESTOBJECT)_m.exe
endif
slave  : $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_$(TARGET_FLAG)$(EXEFILE_EXTENSION)

# COMPILE USERCODE / STUBS
#
$(S_UC_OBJECT) : $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_usr.cpp
	$(S_CC) $(S_INCLUDES) $(S_COMP_OPTIONS) -o "$@" $<

$(S_STUB_OBJECT) : $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_stubs.cpp
	$(S_CC) $(S_INCLUDES) $(S_COMP_OPTIONS) -o "$@" $<


#
# COMPILE/LINK MASTER
#

include $(M_COMM_MAKE)


#
# COMPILE SLAVE COMMUNICATION MODULES

include $(S_COMM_MAKE)


#
#
# COMPILE/LINK SLAVE
#
$(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_s$(OBJFILE_EXTENSION) : $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_s.cpp
	$(info ***** Compiling Slave *****)
	$(S_CC) -o "$@" $(S_COMP_OPTIONS) $(S_INCLUDES) $<

#
# Handle Board Files
#
ifeq ("true","$(USE_BOARD_FILES)")
ifeq ("","$(INITOBJDIR)")
$(error Attribute 'Use Board Files' is set to true but 'InitObjDir' is empty.)
endif
INITOBJDIR := $(shell ts_pathname -s '$(INITOBJDIR)')
ifeq ("","$(INITSRCDIR)")
START_OBJS := $(wildcard $(INITOBJDIR)/*$(OBJFILE_EXTENSION))
else
INITSRCDIR := $(shell ts_pathname -s '$(INITSRCDIR)')
START_OBJS := $(patsubst $(INITSRCDIR)/%.c,$(INITOBJDIR)/%$(OBJFILE_EXTENSION),$(wildcard $(INITSRCDIR)/*.c)) \
              $(patsubst $(INITSRCDIR)/%.cpp,$(INITOBJDIR)/%$(OBJFILE_EXTENSION),$(wildcard $(INITSRCDIR)/*.cpp)) \
              $(patsubst $(INITSRCDIR)/%.s,$(INITOBJDIR)/%$(OBJFILE_EXTENSION),$(wildcard $(INITSRCDIR)/*.s))

$(INITOBJDIR)/%$(OBJFILE_EXTENSION) : $(INITSRCDIR)/%.c
	$(S_CC) -o "$@" $(S_COMP_OPTIONS) $(COMM_INCLUDES) $< -I"$(INITOBJDIR)"

$(INITOBJDIR)/%$(OBJFILE_EXTENSION) : $(INITSRCDIR)/%.cpp
	$(S_CC) -o "$@" $(S_COMP_OPTIONS) $(COMM_INCLUDES) $< -I"$(INITOBJDIR)"

$(INITOBJDIR)/%$(OBJFILE_EXTENSION) : $(INITSRCDIR)/%.s
	$(S_CC) -o "$@" $(S_COMP_OPTIONS) $(COMM_INCLUDES) $< -I"$(INITOBJDIR)"
endif
ifeq ("","$(START_OBJS)")
$(warning Attribute 'Use Board Files' is set to true but no objects were found.)
endif
endif


#
# Objects to be delete after the build process.
#
TS_INTERMEDIATE_OBJS :=  $(S_OBJECTS) $(BUILTIN_OBJ) $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_s$(OBJFILE_EXTENSION) \
                         $(S_SRC_OBJECTS) $(S_UC_OBJECT) $(S_STUB_OBJECT)
.INTERMEDIATE: $(TS_INTERMEDIATE_OBJS)


$(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_$(TARGET_FLAG)$(EXEFILE_EXTENSION): $(START_OBJS) $(TS_INTERMEDIATE_OBJS)
	$(if $(START_OBJS),$(info ***** Linking Slave with Board Files *****),$(info ***** Linking Slave *****))
	($(S_LINK) $(S_LINK_OPTIONS) $(USER_LINK_OPTIONS) $(START_OBJS) $(TS_INTERMEDIATE_OBJS) -o $@ && $(S_BIN2HEX) $@) > link.msg 2>&1 || (cat link.msg && @exit 1)
ifneq ($(S_GenerateBuiltinData),)
	@-ts_rm -f "$(MODULE_PATH)"/tslows_builtin.*
	@-ts_cp -f "$@" "$(MODULE_PATH)\ts_$(TESTOBJECT)_$(TARGET_FLAG).X\dist\default\production"
	$(info *)
	$(info * Generated target binary '$(shell ts_pathname -w $(MODULE_PATH)\ts_$(TESTOBJECT)_$(TARGET_FLAG)$(EXEFILE_EXTENSION))' containing builtin data.)
	$(info *)
endif

#####################################################################
#END_TARGETS
#####################################################################
